@@ -27,10 +27,10 @@
 # input fasta files with proteins
 # output matrix with orthologous proteins
 # 
-# @author Marcus Lechner
+# @authors Marcus Lechner, Clemens Elias Thoelken
 # @email lechner@staff.uni-marburg.de
 # @company University of Maruburg
-# @date 2016-04-22
+# @date 2016-08-26
 #
 ##########################################################################################
 
@@ -47,7 +47,7 @@ use Thread::Queue;
 ##########################################################################################
 # Variables
 ##########################################################################################
-our $version = "5.13";
+our $version = "5.15";
 our $step = 0;		# 0/1/2/3	-> do all / only apply step 1 / only apply step 2 / only apply step 3
 our $verbose = 1;	# 0/1		-> don't / be verbose
 our $debug = 0;		# 0/1		-> don't / show debug data
@@ -172,6 +172,14 @@ our $simtable = "$project.proteinortho";			# Output file graph
 our $syntable = "$project.poff";					# Output file synteny
 our $desctable = "$project.descriptions";			# Output file seq descriptions
 
+
+if (-e "$project.proteinortho") {
+	print STDERR "!!!\nWarning: Data files for project '$project' already exists. Previous output files might be overwritten.\n";
+	print STDERR "Press CTRL + C to prevent me from proceeding)\nWaiting for 10 seconds...\n!!!\n";
+	sleep(10);
+	print STDERR "\nWell then, proceeding...\n\n";
+}
+
 ##########################################################################################
 # Run
 ##########################################################################################
@@ -211,7 +219,7 @@ if ($step == 0 || $step == 3) {
 
 
 ##########################################################################################
-# Funktions
+# Functions
 ##########################################################################################
 sub clean {
 	print STDERR "Removing temporary files...\n";
@@ -273,8 +281,9 @@ sub cluster {
 }
 
 sub print_header {
-	print STDERR "Proteinortho with PoFF version $version - An orthology detection tool\n",
-	             "******************************************************************\n";
+	print STDERR "*****************************************************************\n",
+		     "Proteinortho with PoFF version $version - An orthology detection tool\n",
+	             "*****************************************************************\n";
 }
 
 sub print_usage {
@@ -282,8 +291,8 @@ print STDERR << "JUS";
 
 Usage: proteinortho5.pl [OPTIONS] FASTA1 FASTA2 [FASTA...]
 Options: -e=          E-value for blast [default: 1e-05]
-         -p=          blast program {blastn|blastp|blastn+|blastp+}
-                      [default: blastp+]
+         -p=          blast program [default: blastp+]
+                      {blastn|blastp|tblastx|blastn+|blastp+|tblastx+}
          -project=    prefix for all result file names [default: myproject]
          -temp=       path for temporary files [default: working directory]
          -synteny     activate PoFF extension to separate similar sequences
@@ -320,7 +329,7 @@ Options: -e=          E-value for blast [default: 1e-05]
          In case jobs should be distributed onto several machines, use
          -startat=    File number to start with (default: 0)
          -stopat=     File number to end with (default: -1)
- 
+
 JUS
 }
 
@@ -345,7 +354,7 @@ sub set_threads_per_process {
 	lock($jobs_done);
 	my $willdo = ($jobs_todo-$jobs_done+$_[0]);
 
-	if ($debug) {	
+	if ($debug) {
 		print STDERR "\nTODO: $jobs_todo DONE: $jobs_done Running: $_[0] -> $willdo\n";
 	}
 
@@ -411,7 +420,7 @@ sub workerthread {
 			# If there is nothing
 			unless (defined($job)) {
 				# Check if more jobs need to be done
-	
+
 				{
 				lock($jobs_done);							# 	Productive
 				if ($jobs_done >= $jobs_todo) {						# 	Productive
@@ -422,7 +431,7 @@ sub workerthread {
 				}}
 				# If so, wait
 					if ($debug) {print STDERR "Thread $thread_id\tis seeking work ($jobs_done / $jobs_todo)\n";}
-				sleep(1);			
+				sleep(1);
 			}
 			else {
 				# Parse job
@@ -440,7 +449,7 @@ sub workerthread {
 		# Work
 		&set_threads_per_process(scalar(threads->list()));
 		my $result_ij = &blast($file_i,$file_j);
-		
+
 		my $result_ji;
 		if ($file_i eq $file_j) {
 			# One run is enough (selfblast)
@@ -476,9 +485,9 @@ sub workerthread {
 			my $synt_stats = qx($cmd);
 			chomp $synt_stats;
 			$synt_stats =~ s/#.+\n//;
-			
+
 			# Reverse mapping of full gene ids, two seperate maps in case of overlapps in short ids
-			my %full_id_map_i;				
+			my %full_id_map_i;
 			my %full_id_map_j;
 			foreach (sort keys %reciprocal_matches) {
 				my ($a, $b) = split(" ",$_);
@@ -540,7 +549,7 @@ sub workerthread {
 
 			{
 			lock($syn_lock);
-			
+
 			open(SYN,">>$syngraph") || die("Could not open file '$syngraph': $!");
 			print SYN "# $short_file_j\t$short_file_i\n";
 			print SYN "# Scores: $synt_stats\n";
@@ -649,7 +658,7 @@ sub adaptive_best_blast_matches {
 
 sub synteny_matches {
 	my %reciprocal_matches = %{(shift)};
-	my $file_i = shift; 
+	my $file_i = shift;
 	my $file_j = shift;
 
 	# Get order for both species (same hash as ids are non overlapping)
@@ -660,11 +669,11 @@ sub synteny_matches {
 		my %coords = %{&read_details($file)};
 		my $counter = 0;
 		# Number them according to their order
-		foreach my $id (sort 
+		foreach my $id (sort
 			{
 				my @a = split("\t",$coords{$a});
 				my @b = split("\t",$coords{$b});
-	
+
 #				#chr strand pos
 #				if ($a[0] ne $b[0]) {return $a[0] cmp $b[0];}
 #				if ($a[1] ne $b[1]) {return $a[1] cmp $b[1];}
@@ -700,7 +709,7 @@ sub synteny_matches {
 		if ($a[1] eq $b[1]) 	{$output .= "1\t";}	# Same strand?
 		else 			{$output .= "-1\t";}
 		my $score = (&edgeweight($values[0])+&edgeweight($values[2]))/2;	# Score made from e-values
-		$output .= $score."\n";	
+		$output .= $score."\n";
 	}
 
 	# Check multis
@@ -755,7 +764,7 @@ sub print_blast_stats {
 		if ($jobs_todo == 0) {die("Nothing to do. This should not happen!");}
 		lock($jobs_done);
 		my $percent = int($jobs_done/$jobs_todo*10000)/100;
-		print STDERR "\r                                                                               ";		
+		print STDERR "\r                                                                               ";
 		print STDERR "\rRunning blast analysis: $percent% ($jobs_done/$jobs_todo)";
 	}
 }
@@ -764,16 +773,16 @@ sub match {
 	my %length = %{(shift)};
 	my @i = @{(shift)};
 	my @j = @{(shift)};
-	
+
 	my %legal_i = &get_legal_matches(\%length,@i);
 	my %legal_j = &get_legal_matches(\%length,@j);
-	
+
 	return &get_reciprocal_matches(\%legal_i,\%legal_j);
 }
 
 sub get_reciprocal_matches {
 	my %i = %{(shift)};
-	my %j = %{(shift)}; 
+	my %j = %{(shift)};
 	my %reciprocal;
 
 	foreach (keys %i) {
@@ -795,7 +804,10 @@ sub get_legal_matches {
 	my %result;
 	foreach (@_) {
 		my ($query_id,$subject_id,$local_identity,$alignment_length,$mismatches,$openings,$query_start,$query_end,$subject_start,$subject_end,$local_evalue,$local_bitscore) = split(/\s+/);
-	
+
+		if ($debug) {print STDERR "?$query_id -> $subject_id ($local_evalue,$local_bitscore)\n";}
+
+
 		# Bug tracking
 		unless (defined($length{$query_id})) 	{print STDERR "ERROR: Query gene ID '$query_id' is present in blast output but was not present in FASTA input! Skipping line.\n"; next;}
 		unless (defined($length{$subject_id})) 	{print STDERR "ERROR: Subject gene ID '$subject_id' is present in blast output but was not present in FASTA input! Skipping line.\n"; next;}
@@ -803,10 +815,12 @@ sub get_legal_matches {
 		## Check for criteria
 		# Well formatted
 		if (!defined($local_bitscore)) 							{next;}
+
 		# Percent identity
 		if (!$twilight && $local_identity < $identity) 					{next;}
 		if ( $twilight && $local_identity < &identitybylength($alignment_length))	{next;} 
 		# Min. length
+		if ($blastmode eq "tblastx+" || $blastmode eq "tblastx") {$alignment_length *= 3;}
 		if ($alignment_length < $length{$query_id}*($coverage/100)+0.5) 		{next;}
 		if ($alignment_length < $length{$subject_id}*($coverage/100)+0.5) 		{next;}
 		# It hit itself (only during selfblast)
@@ -815,11 +829,12 @@ sub get_legal_matches {
 		# Similar hits? Take the better one
 		if (defined($result{"$query_id $subject_id"})) {
 			my ($remote_evalue, $remote_bitscore) = split(" ",$result{"$query_id $subject_id"});
-			if ($local_evalue > $remote_evalue) {next;}			
+			if ($local_evalue > $remote_evalue) {next;}
 			if ($local_bitscore < $remote_bitscore) {next;}
 		}
 
 		# Store data
+		if ($debug) {print STDERR "!$query_id -> $subject_id ($local_evalue,$local_bitscore)\n";}
 		$result{"$query_id $subject_id"} = "$local_evalue $local_bitscore";
 	}
 
@@ -860,9 +875,13 @@ sub generate_indices {
 
 sub blast {
 	my $command = "";
-	if 	($blastmode eq "blastp" || $blastmode eq "blastn") 	{lock($threads_per_process); $command = $blastpath."blastall -a $threads_per_process -d '$_[0]' -i '$_[1]' -p $blastmode -m8 -e $evalue $blastOptions";}
+	if 	($blastmode eq "blastp" || $blastmode eq "blastn" || $blastmode eq "tblastx") {
+			lock($threads_per_process);
+			$command = $blastpath."blastall -a $threads_per_process -d '$_[0]' -i '$_[1]' -p $blastmode -m8 -e $evalue $blastOptions";
+	}
 	elsif	($blastmode eq "blastp+") 				{lock($threads_per_process); $command = $blastpath."blastp -num_threads $threads_per_process -db '$_[0]' -query '$_[1]' -evalue $evalue -outfmt 6 $blastOptions";}
 	elsif	($blastmode eq "blastn+") 				{lock($threads_per_process); $command = $blastpath."blastn -num_threads $threads_per_process -db '$_[0]' -query '$_[1]' -evalue $evalue -outfmt 6 $blastOptions";}
+	elsif	($blastmode eq "tblastx+") 				{lock($threads_per_process); $command = $blastpath."tblastx -num_threads $threads_per_process -db '$_[0]' -query '$_[1]' -evalue $evalue -outfmt 6 $blastOptions";}
 	else	{die("This should not happen!");}
 
 	my $a = $_[0];
@@ -894,7 +913,7 @@ sub readFile {
 }
 
 sub check_blast {
-	if ($blastmode eq "blastp+" || $blastmode eq "blastn+") {
+	if ($blastmode eq "blastp+" || $blastmode eq "blastn+" || $blastmode eq "tblastx+") {
 		my $tmp = $blastmode;
 		$tmp =~ s/\+//g;
 		my $cmd = $blastpath."$tmp -h";
@@ -905,15 +924,15 @@ sub check_blast {
 
 			# Commands
 			if 	($blastmode eq "blastp+") {$makedb = $blastpath."makeblastdb -dbtype prot -in";}
-			elsif 	($blastmode eq "blastn+") {$makedb = $blastpath."makeblastdb -dbtype nucl -in";}
+			elsif 	($blastmode eq "blastn+" || $blastmode eq "tblastx+") {$makedb = $blastpath."makeblastdb -dbtype nucl -in";}
 			else	{die("This should not happen!");}
-  
+
 			print STDERR "Detected NCBI BLAST version $blastversion\n";
 			return;
 		}
 		&Error("Failed to detect '$blastmode'! Tried to call '$tmp'.");
 	}
-	elsif ($blastmode eq "blastp" || $blastmode eq "blastn") {
+	elsif ($blastmode eq "blastp" || $blastmode eq "blastn" || $blastmode eq "tblastx") {
 		my $cmd = $blastpath."blastall";
 		my @blastv = qx($cmd);
 		foreach (@blastv) {
@@ -922,6 +941,7 @@ sub check_blast {
 				$blastversion = $1;
 				if 	($blastmode eq "blastp") {$makedb = $blastpath."formatdb -p T -o F -i";}
 				elsif 	($blastmode eq "blastn") {$makedb = $blastpath."formatdb -p F -o F -i";}
+				elsif 	($blastmode eq "tblastx") {$makedb = $blastpath."formatdb -p F -o F -i";}
 				else	{die("This should not happen!");}
 
 				print STDERR "Detected NCBI BLAST version $blastversion\n";
@@ -946,7 +966,7 @@ sub check_files {
 
 sub convertNCBI {
 	my $long_id = shift;
-	$long_id =~ s/\|$//g;					
+	$long_id =~ s/\|$//g;
 	my @tmp = split(/\|/,$long_id);	# take the last column for NCBI format like patterns (e.g. gi|158333234|ref|YP_001514406.1|)
 	return pop(@tmp);
 }
