@@ -24,31 +24,35 @@ using namespace std;
 struct protein {vector<unsigned int> edges; unsigned int species_id; string full_name;};
 
 // Functions
-float string2float(string);
+double string2double(string);
 void tokenize(const string& , vector<string>& , const string&);
 void parse_file(string);
 void remove_edge_between(const unsigned int, const unsigned int);
 void remove_edge(protein&, const unsigned int);
-float getConnectivity(vector<unsigned int>&);
+double getConnectivity(vector<unsigned int>&);
 void clear_edges(vector<unsigned int>&);
 void partition_graph(void);
-void print_group(vector<unsigned int>& , float );
+void print_group(vector<unsigned int>& , double );
 void print_header(void);
+void sort_species(void);
 vector<unsigned int> get_deg_one (vector<unsigned int>& );
 
 // Parameters
 bool param_verbose 		= false;
-float param_con_threshold 	= 0.1;
+double param_con_threshold 	= 0.1;
 string param_rmgraph            = "remove.graph";
+unsigned int min_iter 		= 100;		// min number of alg con iterations
+unsigned int max_iter 		= 10000;	// max number of alg con iterations
 
 // Globals
 unsigned int species_counter = 0;	// Species
 unsigned int protein_counter = 0;	// Proteins
 vector<string> species;			// Number -> Name
 vector<protein> graph;			// Graph containing all protein data
-float last_stat = 0;			// For progress stats
+double last_stat = 0;			// For progress stats
 unsigned int edges = 0;			// number of edges
 ofstream graph_clean;			// File to store graph data
+vector<int> reorder_table;		// Tells how proteins/species must be sorted
 
 // TMP Globals
 map<string,int> species2id;		// Name -> Number
@@ -65,7 +69,7 @@ void printHelp() {
 	cerr << "" << endl;
 	cerr << "Usage:   proteinortho5_clustering [OPTIONS] graph_files..." << endl;
 	cerr << "Options: -verbose        report progress" << endl;
-	cerr << "         -conn FLOAT     threshold for connectivity [0.1]" << endl;
+	cerr << "         -conn double     threshold for connectivity [0.1]" << endl;
 	cerr << "         -rmgraph STRING output file for graph" << endl;
 }
 
@@ -86,13 +90,13 @@ int main(int argc, char *argv[]) {
 		}
 		else if (parameter == "-verbose") {
 			paras++;
-			if (string2float(string(argv[paras]))) {
+			if (string2double(string(argv[paras]))) {
 				param_verbose = true;
 			}
 		}
 		else if (parameter == "-conn") {
 			paras++;
-			param_con_threshold = string2float(string(argv[paras]));
+			param_con_threshold = string2double(string(argv[paras]));
 		}
 		else if (parameter == "-rmgraph") {
 			paras++;
@@ -117,6 +121,9 @@ int main(int argc, char *argv[]) {
 	// Stats
 	if (param_verbose) cerr << species_counter << " species" << endl << protein_counter << " paired proteins" << endl << edges << " bidirectional edges" << endl;
 
+	// Prepare sort of output
+	sort_species();
+
 	// Write output header
 	print_header();							
 
@@ -157,10 +164,25 @@ void print_edgelist(protein& protein) {
 ///////////////////////////////////////////////////////////
 // Output
 ///////////////////////////////////////////////////////////
+// Sort
+void sort_species(void) {
+	reorder_table.reserve(species_counter);
+	vector<string> species_sorted (species_counter);
+	copy(species.begin(), species.end(), species_sorted.begin());
+	sort(species_sorted.begin(), species_sorted.end());
+	// find new locations (not efficent but list is small)	
+	for (unsigned int i = 0; i < species_counter; i++) {
+		for (unsigned int j = 0; j < species_counter; j++) {
+			if (species[i] == species_sorted[j]) {reorder_table[j] = i; continue;}
+		}
+	}
+}
+
+
 // Progress stats
-void stats(float i, float size) {
+void stats(double i, double size) {
 	if (!param_verbose) return;
-	float stat = float(i/size*100);
+	double stat = double(i/size*100);
 	if (last_stat * 1.01 < stat) {
 		last_stat = stat;
 		cerr << "\r" << "                          ";
@@ -173,13 +195,13 @@ void stats(float i, float size) {
 void print_header() {
 	cout << "# Species\tGenes\tAlg.-Conn.";
 	for (unsigned int i = 0; i < species_counter; i++) {
-		cout << "\t" << species[i];
+		cout << "\t" << species[reorder_table[i]];
 	}
 	cout << endl;
 }
 
 // Group formatting
-void print_group(vector<unsigned int>& nodes, float connectivity) {
+void print_group(vector<unsigned int>& nodes, double connectivity) {
 	vector<string> line(species_counter,"*");	// Output vector
 
 	unsigned int species_number = 0;
@@ -197,8 +219,23 @@ void print_group(vector<unsigned int>& nodes, float connectivity) {
 	}
 
 	cout << species_number << "\t" << nodes.size() << "\t" << setprecision (3) << connectivity;
-	for (vector<string>::iterator it=line.begin() ; it < line.end(); it++) {
-		cout << "\t" << *it;
+
+	// List group data
+	for (unsigned int i = 0; i < species_counter; i++) {
+		
+		// sort line if multi protein
+		vector<string> fields;
+		tokenize(line[reorder_table[i]], fields, ",");
+		if (fields.size() > 1) {
+			sort(fields.begin(), fields.end());
+			line[reorder_table[i]] = fields[0];
+			for (unsigned int k = 1; k < fields.size(); k++) {
+				line[reorder_table[i]].append(","+fields[k]);
+			}
+		}
+
+		// output
+		cout << "\t" << line[reorder_table[i]];
 	}
 	cout << endl;
 }
@@ -251,7 +288,7 @@ void partition_graph() {
 		}
 
 		// Connectivity analysis
-		float connectivity = getConnectivity(current_group);
+		double connectivity = getConnectivity(current_group);
 
 		if (connectivity < param_con_threshold && current_group.size() > 3) {
 			// Split groups is done in getConnectivity function
@@ -298,10 +335,10 @@ vector<unsigned int> get_deg_one (vector<unsigned int>& nodes) {
 }
 
 // Generate random vector x of size size
-vector<float> generate_random_vector(const unsigned int size) {
-	vector<float> x(size);
+vector<double> generate_random_vector(const unsigned int size) {
+	vector<double> x(size);
 	for (unsigned int i = 0; i < size; i++) {
-	  x[i] = (float)(rand() % 999+1)/1000;	// 1 bis 99
+	  x[i] = (double)(rand() % 999+1)/1000;	// 1 bis 99
 		// at least one value must be different from the others but still within 0 and 1
 		if (i > 0 && x[i] == x[i-1]) {
 			x[i] /= 3;
@@ -312,10 +349,10 @@ vector<float> generate_random_vector(const unsigned int size) {
 }
 
 // Generate random vector x of size size
-vector<float> generate_random_vector_old(const unsigned int size) {
-	vector<float> x(size);
+vector<double> generate_random_vector_old(const unsigned int size) {
+	vector<double> x(size);
 	for (unsigned int i = 0; i < size; i++) {
-	  x[i] = (float)rand()/RAND_MAX;
+	  x[i] = (double)rand()/RAND_MAX;
 		// at least one value must be different from the others but still within 0 and 1
 		if (i > 0 && x[i] == x[i-1]) {
 			x[i] += 0.1;
@@ -329,8 +366,8 @@ vector<float> generate_random_vector_old(const unsigned int size) {
 }
 
 // determine new X, Formula (1)
-vector<float> get_new_x(vector<float> x, vector<unsigned int>& nodes, map<unsigned int,unsigned int>& mapping) {
-	vector<float> x_new(x.size());
+vector<double> get_new_x(vector<double> x, vector<unsigned int>& nodes, map<unsigned int,unsigned int>& mapping) {
+	vector<double> x_new(x.size());
 	// go through all nodes (rows of A)
 	for (unsigned int i = 0; i < nodes.size(); i++) {
 		unsigned int node = nodes[i];	// node x is node y here
@@ -348,17 +385,17 @@ vector<float> get_new_x(vector<float> x, vector<unsigned int>& nodes, map<unsign
 }
 
 // Make vector x orthogonal to 1, Formula (2)
-vector<float> makeOrthogonal(vector<float> x) {
-	float sum = 0;
+vector<double> makeOrthogonal(vector<double> x) {
+	double sum = 0;
 	for (unsigned int i = 0; i < x.size(); i++) {sum += x[i];}
-	float average = sum/x.size();
+	double average = sum/x.size();
 	for (unsigned int i = 0; i < x.size(); i++) {x[i] -= average;}
 	return x;
 }
 
 // Normalize vector x, Formula (4)
-vector<float> nomalize(vector<float> x, float *length) {
-	float sum = 0;
+vector<double> nomalize(vector<double> x, double *length) {
+	double sum = 0;
 	for (unsigned int i = 0; i < x.size(); i++) {sum += x[i]*x[i];}
 	*length = sqrt(sum);
 	if (*length == 0) {*length = 0.000000001;}	// ATTENTION not 0!
@@ -367,7 +404,7 @@ vector<float> nomalize(vector<float> x, float *length) {
 }
 
 // Qx, Formula (5)
-vector<float> getY(float max_degree, vector<float> x_hat, vector<float> x_new, vector<unsigned int>& nodes){
+vector<double> getY(double max_degree, vector<double> x_hat, vector<double> x_new, vector<unsigned int>& nodes){
 	// (2*maxdeg - grad_node_i ) * x_hat_i + new_x_i
 	for (unsigned int i = 0; i < x_hat.size(); i++) {
 		x_hat[i] *= (2*max_degree - graph[nodes[i]].edges.size());
@@ -438,17 +475,27 @@ void removeExternalEdges_old(map<unsigned int,bool>& a) {
 }
 
 // Split connected component according to eigenvector
-void splitGroups(vector<float>& y, vector<unsigned int>& nodes, map<unsigned int,unsigned int> mapping){
+void splitGroups(vector<double>& y, vector<unsigned int>& nodes){
+
+//	cerr << "Mission to split groups:" << endl << "####################" << endl;		///
+
+//	cerr << "Vector (" << nodes.size() << ")" << endl;
+//	for (unsigned int i = 0; i < nodes.size(); i++) {
+//		cerr << graph[nodes[i]].full_name << "\t" << nodes[i] << "\t" << y[i] << endl;
+//	}
+
+
 	// Remove tree like structures in the beginning
-	vector<unsigned int> one = get_deg_one(nodes);
-	if (one.size() > 0) {
-		map<unsigned int,bool> tree;
-		for (unsigned int i = 0; i < one.size(); i++) {
-			{tree[one[i]] = true;}
-		}
-		removeExternalEdges(tree);
-		return;
-	}
+//	vector<unsigned int> one = get_deg_one(nodes);
+//	if (one.size() > 0) {
+//		cerr << "Tree " << endl;		///
+//		map<unsigned int,bool> tree;
+//		for (unsigned int i = 0; i < one.size(); i++) {
+//			{tree[one[i]] = true;}
+//		}
+//		removeExternalEdges(tree);
+//		return;
+//	}
 
 	// Store data about two groups
 	map<unsigned int,bool> groupA, groupB;
@@ -469,7 +516,7 @@ void splitGroups(vector<float>& y, vector<unsigned int>& nodes, map<unsigned int
 	removeExternalEdges(groupB);
 }
 
-float getConnectivity(vector<unsigned int>& nodes) {
+double getConnectivity(vector<unsigned int>& nodes) {
 	// Special cases hotfix
 	if (nodes.size() == 2) {return 1;}
 
@@ -478,7 +525,7 @@ float getConnectivity(vector<unsigned int>& nodes) {
 		// fully connected
 		if (min.size() == 0) {return 1;}
 		// not
-		else {return 0.667;}
+		else {return 1/3;}
 	}
 	// Hotfix end
 
@@ -490,36 +537,40 @@ float getConnectivity(vector<unsigned int>& nodes) {
 	for (unsigned int i = 0; i < nodes.size(); i++) {mapping[nodes[i]] = i;}
 	
 	// Init randomized x 
-	vector<float> x = generate_random_vector(nodes.size());
+	vector<double> x = generate_random_vector(nodes.size());
 
 	// Orthogonalize + normalize vector + get initial lenght
-	float last_length, current_length = 0;
-	vector<float> x_hat = makeOrthogonal(x);
-	vector<float> norm = nomalize(x_hat, &last_length);
+	double last_length, current_length = 0;
+	vector<double> x_hat = makeOrthogonal(x);
+	vector<double> norm = nomalize(x_hat, &last_length);
 
 	// Repeat until difference < epsilon
-	unsigned int iter = 1000;							// max 1000 iterations (catch huge clustering issues)
-	while(iter-- != 0) { 
+	unsigned int iter = 0;							// catch huge clustering issues by keeping track here
+	while(iter++ < max_iter) { 
 		last_length = current_length;
 		// Get a new x
 		x = get_new_x(norm, nodes, mapping);
 		// Get y
-		vector<float> y = getY(max_degree,norm,x,nodes);
+		vector<double> y = getY(max_degree,norm,x,nodes);
 		// Orthogonalize
 		x_hat = makeOrthogonal(y);
 		// Get lenght (lambda) & normalize vector
 		norm = nomalize(x_hat, &current_length);
-		if (abs(current_length-last_length) < 0.0001 && iter >= 20) break;	// min 20 iterations (prevent convergence by chance)
+///		cerr << "I " << iter << ": " << abs(current_length-last_length) << endl; 
+		if (abs(current_length-last_length) < 0.00001 && iter >= min_iter) break;	// min 100 iterations (prevent convergence by chance), converge to 1e-6
 	}
-	//	cerr << nodes.size() << " nodes done after " << iter << " iterations" << endl;
+///		cerr << nodes.size() << " nodes done after " << iter << " iterations" << endl;
 
-	float connectivity = (-current_length+2*max_degree)/(nodes.size());
+	double connectivity = (-current_length+2*max_degree)/(nodes.size());
 
 //	cerr << nodes.size() << " " << connectivity << endl;
 
+///	if (connectivity > 1) {cerr << "DIE" << endl; throw "XX";}
+
 	// Split groups if connectivity is too low, remove tree like structures that might have arosen
 	if (connectivity < param_con_threshold) {
-		splitGroups(x_hat, nodes, mapping);
+///		cerr << "Conn: " << connectivity << endl; ///
+		splitGroups(x_hat, nodes);
 	}
 	
 	return connectivity;
@@ -628,10 +679,10 @@ void parse_file(string file) {
 ///////////////////////////////////////////////////////////
 // Misc functions
 ///////////////////////////////////////////////////////////
-// Convert string to float
-float string2float(string str) {
+// Convert string to double
+double string2double(string str) {
 	istringstream buffer(str);
-	float value;
+	double value;
 	buffer >> value;
 	return value;
 }
